%% C. Cantore (2021)
%% Tractable Heterogeneous Agents Models 
%% CIMS Summer School - University of Surrey


%%===================================================================================================
% Reproduces figures in session 4 on Moderate constant Liquidity based on material from Vincent Sterk
%%===================================================================================================
 




clc
close all
clear


% parameters
p.Bbar=0;                 % aggregate amount of liquidity
p.Rbar=(1.01)^(1/4);      % steady-state target interest rate (quarterly) 
p.Nbar=1;                 % steady-state target labor supply
p.beta=0.99;              % subjective discount factor
p.eps=9;                  % elasticity of subst. goods varieties
p.phi=1;                  % inverse Frische elasticity of labor supply
p.sig=1;                  % coefficient of risk aversion 
p.ksi=1.5;                % coefficient Taylor rule (inflation)
p.pdur=4;                 % average price duration (in quarters)
p.calvo=1-1/p.pdur;       % Calvo probability of adjustment
p.phiR=p.calvo*(p.eps-1)/((1-p.calvo)*(1-p.beta*p.calvo)); % implied Rotemberg parameter (exploit first-order equivalence)
p.peu=0.03;               % job loss prob
p.pue=0.6;                % job finding prob 
p.u=p.peu/(p.pue+p.peu);  % unemployment rate
p.rhoA=0.9;               % persistence TFP shock
p.rhoR=0.5;               % persistence monetary policy shock


% parameters zero-liquidity version
p.kappa_zl=(p.eps-1)/p.eps;  % normalize kappa to ensure Ne=1 --> kappa=w=(eps-1)/eps) (use labour supply condition and Phillips Curve)
p.theta_zl=((1/p.beta/p.Rbar-1+p.peu)/p.peu).^(-1/p.sig);

K=75;                     % truncation cohort (quarters)
% compute population shares cohorts
p.psi(1) = p.u * p.pue;                                                 % population share employed for 1 quarter 
for i=2:K-1
    p.psi(i)=p.psi(i-1) * (1 - p.peu);                                  % population share employed for i quarters
end
p.psi(K)=1-sum(p.psi(1:K-1))-p.u; 
save params p;

iter=1;
Bbarvec=0;
stepsize=0.02;
check_condition=1;

while check_condition>0
dynare model_moderate_liq_calib.mod noclearall;

% save steady state
for i=1:K+1
p.C_ss(i)=oo_.steady_state(i);
p.B_ss(i)=oo_.steady_state(i+K+1);
p.N_ss(i)=oo_.steady_state(i+2*(K+1));
end
p.theta=oo_.steady_state(3*(K+1)+1);
p.kappa=oo_.steady_state(3*(K+1)+2);
save params p;
check_condition=(p.C_ss(1)+p.Rbar*p.B_ss(end))^(-p.sig)-p.beta*p.Rbar*((1-p.pue)*p.C_ss(1)^(-p.sig)+p.pue*p.C_ss(2)^(-sig));
checkvec(iter)=check_condition;
if check_condition>0
thetavec(iter)= p.theta;
kappavec(iter)= p.kappa;
Bbarvec(iter) = p.Bbar;
dynare model_moderate_liq.mod noclearall;
IRF.MLHANK.eA.Y(iter,:)=Y_eA;
IRF.MLHANK.eA.PI(iter,:)=PI_eA;
IRF.MLHANK.eA.A(iter,:)=A_eA;
IRF.MLHANK.eR.Y(iter,:)=Y_eR;
IRF.MLHANK.eR.PI(iter,:)=PI_eR;
IRF.MLHANK.eR.zR(iter,:)=zR_eR;
B_ss_mat(iter,:)=p.B_ss;
temp_mat(iter,:)=cumsum([p.u p.psi]);
iter=iter+1;
p.Bbar=p.Bbar+stepsize;
save params p;
end
end


figure (1)
plot(Bbarvec,thetavec,'-o','Linewidth',1.5,'MarkerSize',4)
xlabel('aggregate liquidity (B)');ylabel('home production (\vartheta)')

figure(2)
plot(temp_mat',B_ss_mat','Linewidth',1.5);
title('distribution liquid assets')
xlabel('CDF');ylabel('B(i)')
ylim([-0.01 0.2])
legend('B=0','B=0.02','B=0.04','B=0.06','B=0.08','B=0.10','B=0.12','B=0.14');

figure(3)
subplot(2,2,1)
plot(IRF.MLHANK.eA.Y','Linewidth',1.5);hold
title('Output (Y)');ylabel('%');xlabel('quarter')
subplot(2,2,2)
plot(IRF.MLHANK.eA.PI','Linewidth',1.5);hold
title('Inflation (PI)');ylabel('%');xlabel('quarter')
subplot(2,2,3)
plot(IRF.MLHANK.eA.A','Linewidth',1.5);hold
title('TFP (A)');ylabel('%');xlabel('quarter')

figure(4)
subplot(2,2,1)
plot(IRF.MLHANK.eR.Y','Linewidth',1.5);hold
title('Output (Y)');ylabel('%');xlabel('quarter')
subplot(2,2,2)
plot(IRF.MLHANK.eR.PI','Linewidth',1.5);hold
title('Inflation (PI)');ylabel('%');xlabel('quarter')
subplot(2,2,3)
plot(IRF.MLHANK.eR.zR','Linewidth',1.5);hold
title('policy shock (zR)');ylabel('%');xlabel('quarter')
legend('B=0','B=0.02','B=0.04','B=0.06','B=0.08','B=0.10','B=0.12','B=0.14');

% check steady state

