%% C. Cantore (2021)
%% Tractable Heterogeneous Agents Models 
%% CIMS Summer School - University of Surrey


%%=============================================================================================
% Reproduces figures in session 4 on Countercylical risk based on material from Vincent Sterk
%%=============================================================================================
 




clc
clear 
close all



% parameters
p.beta=0.99;              % subjective discount factor
p.eps=9;                  % elasticity of subst. goods varieties
p.phi=1;                  % inverse Frische elasticity of labor supply
p.sig=1;                  % coefficient of risk aversion 
p.ksi=1.5;                % coefficient Taylor rule (inflation)
p.pdur=4;                 % average price duration (in quarters)
p.calvo=1-1/p.pdur;       % Calvo probability of adjustment
p.phiR=p.calvo*(p.eps-1)/((1-p.calvo)*(1-p.beta*p.calvo)); % implied Rotemberg parameter (exploit first-order equivalence)
p.peu=0.03;               % job loss prob
p.pue=0.6;                % job finding prob 
p.u=p.peu/(p.pue+p.peu);  % unemployment rate
p.closs=0.2;              % consumption drop upon job loss in s.s.
p.Rbar=1/(p.beta*(p.peu*((1-p.closs)^(-p.sig))+1-p.peu)); % gross real interest rate in s.s.
p.rhoA=0.9;               % persistence TFP shock
p.rhoR=0.5;               % persistence monetary policy shock
p.rhozpeu=0.9;             % persistence risk shock
p.kappa=(p.eps-1)/p.eps;  % normalize kappa to ensure Ne=1 --> kappa=w=(eps-1)/eps) (use labour supply condition and Phillips Curve)
p.discount=p.beta*p.Rbar*(1-p.peu);
p.zeta=-20;               % feedback parameter endogenous risk 
save params p;


% compute consumption path of (ex post) perpetually employed household
dynare ZLHANK_peu_exo.mod;
scale=0.01/(p.peu*zpeu_epeu(1));

IRF.ZLHANK.epeu.Y=100*scale*Y_epeu;
IRF.ZLHANK.epeu.PI=100*scale*PI_epeu;
IRF.ZLHANK.epeu.peu=scale*zpeu_epeu*p.peu+p.peu;


figure(1)
subplot(1,3,1)
plot(IRF.ZLHANK.epeu.Y,'k-','Linewidth',1.5,'MarkerSize',4);hold
title('Output (Y)');ylabel('%');xlabel('quarter')
subplot(1,3,2)
plot(IRF.ZLHANK.epeu.PI,'k-','Linewidth',1.5,'MarkerSize',4);hold
title('Inflation (PI)');ylabel('%');xlabel('quarter')
subplot(1,3,3)
plot(IRF.ZLHANK.epeu.peu,'k-','Linewidth',1.5,'MarkerSize',4);hold
plot(0*IRF.ZLHANK.epeu.peu+p.peu,'k--','Linewidth',1,'MarkerSize',4);hold
title('p^e^u');ylabel('level');xlabel('quarter');ylim([0 0.05])

%Determinacy with endogenous peu
dynare ZLHANK_peu_endo_determinacy.mod;


dynare ZLHANK_peu_endo.mod;
IRF.ZLHANK_peu_endo.eA.Y=Y_eA;
IRF.ZLHANK_peu_endo.eA.PI=PI_eA;
IRF.ZLHANK_peu_endo.eA.A=A_eA;
IRF.ZLHANK_peu_endo.eA.peu=peu_endo_eA*p.peu+p.peu;
IRF.ZLHANK_peu_endo.eR.Y=Y_eR;
IRF.ZLHANK_peu_endo.eR.PI=PI_eR;
IRF.ZLHANK_peu_endo.eR.zR=zR_eR;
IRF.ZLHANK_peu_endo.eR.peu=peu_endo_eR*p.peu+p.peu;


dynare ZLHANK_reduced.mod;
IRF.ZLHANK.eA.Y=Y_eA;
IRF.ZLHANK.eA.PI=PI_eA;
IRF.ZLHANK.eA.A=A_eA;
IRF.ZLHANK.eR.Y=Y_eR;
IRF.ZLHANK.eR.PI=PI_eR;
IRF.ZLHANK.eR.zR=zR_eR;





figure(2)
subplot(2,2,1)
plot(IRF.ZLHANK.eA.Y,'k-','Linewidth',1.5,'MarkerSize',4);hold
plot(IRF.ZLHANK_peu_endo.eA.Y,'r:','Linewidth',1.5)
title('Output (Y)');ylabel('%');xlabel('quarter')
subplot(2,2,2)
plot(IRF.ZLHANK.eA.PI,'k-','Linewidth',1.5,'MarkerSize',4);hold
plot(IRF.ZLHANK_peu_endo.eA.PI,'r:','Linewidth',1.5)
title('Inflation (PI)');ylabel('%');xlabel('quarter')
subplot(2,2,3)
plot(IRF.ZLHANK.eA.A,'k-','Linewidth',1.5,'MarkerSize',4);hold
plot(IRF.ZLHANK_peu_endo.eA.A,'r:','Linewidth',1.5)
title('TFP (A)');ylabel('%');xlabel('quarter')
%legend('ZL-HANK: exogenous p^e^u','ZL-HANK: endogenous p^e^u')
subplot(2,2,4)
plot(0*IRF.ZLHANK_peu_endo.eA.peu+p.peu,'k-','Linewidth',1.5,'MarkerSize',4);hold
plot(IRF.ZLHANK_peu_endo.eA.peu,'r:','Linewidth',1.5)
title('p^e^u');ylabel('level');xlabel('quarter');ylim([0 0.06])
legend('ZL-HANK: exogenous p^e^u','ZL-HANK: endogenous p^e^u')



figure(3)
subplot(2,2,1)
plot(IRF.ZLHANK.eR.Y,'k-','Linewidth',1.5,'MarkerSize',4);hold
plot(IRF.ZLHANK_peu_endo.eR.Y,'r:','Linewidth',1.5)
title('Output (Y)');ylabel('%');xlabel('quarter')
subplot(2,2,2)
plot(IRF.ZLHANK.eR.PI,'k-','Linewidth',1.5,'MarkerSize',4);hold
plot(IRF.ZLHANK_peu_endo.eR.PI,'r:','Linewidth',1.5)
title('Inflation (PI)');ylabel('%');xlabel('quarter')
subplot(2,2,3)
plot(IRF.ZLHANK.eR.zR,'k-','Linewidth',1.5,'MarkerSize',4);hold
plot(IRF.ZLHANK_peu_endo.eR.zR,'r:','Linewidth',1.5)
title('policy shock (zR)');ylabel('%');xlabel('quarter')
%legend('ZL-HANK: exogenous p^e^u','ZL-HANK: endogenous p^e^u')
subplot(2,2,4)
plot(0*IRF.ZLHANK_peu_endo.eR.peu+p.peu,'k-','Linewidth',1.5,'MarkerSize',4);hold
plot(IRF.ZLHANK_peu_endo.eR.peu,'r:','Linewidth',1.5)
title('p^e^u');ylabel('level');xlabel('quarter');ylim([0 0.06])
legend('ZL-HANK: exogenous p^e^u','ZL-HANK: endogenous p^e^u')

